import io
import requests
import os
from missav_toolbox.base_downloader import BaseDownloader
from PIL import Image, ImageFont, ImageDraw
from missav_toolbox.type_enum import MovieType, SortBy


class CoverDownloader(BaseDownloader):
    """
    Download movie covers from missav.com and make a collage

    :param save_path: str, the path to save the covers
    :param movie_type: professional, fc2, search
    :param sort_by: ReleaseDate, RecentUpdate, TodayViews, WeeklyViews, MonthlyViews
    """

    def __init__(self, save_path='covers', movie_type=MovieType.professional, sort_by=SortBy.ReleaseDate,
                 keywords=None):
        super().__init__(save_path, movie_type, sort_by, keywords)

        self.count = 0
        self.top_covers = {}
        self.top_ids = {}

    def download_cover(self, movie_title, movie_id):
        # fix the bug that the movie title contains '/?*\'
        movie_title = movie_title.replace('/', '')
        movie_title = movie_title.replace('*', '')
        movie_title = movie_title.replace('?', '')
        movie_title = movie_title.replace('\\', '')

        cover_save_path = os.path.join(self.save_path, f'{movie_title}.jpg')
        if os.path.exists(cover_save_path):
            if self.count < 10:
                with open(cover_save_path, 'rb') as f:
                    self.top_covers[self.count] = f.read()
                self.top_ids[self.count] = movie_id.upper()
                self.count += 1
            return

        print(f'Downloading cover: {movie_title}')

        try:
            movie_cover_link = f'https://fivetiu.com/{movie_id}/cover-n.jpg'
            cover = self.scraper.get(movie_cover_link, proxies=self.proxy, headers=self.headers)
            cover.raise_for_status()
            with open(cover_save_path, 'wb') as f:
                f.write(cover.content)
        except requests.exceptions.HTTPError:
            # try to download low resolution cover
            movie_cover_link = f'https://fivetiu.com/{movie_id}/cover-t.jpg'
            cover = self.scraper.get(movie_cover_link, proxies=self.proxy, headers=self.headers)
            with open(cover_save_path, 'wb') as f:
                f.write(cover.content)

        # get top10 movies
        if self.count < 10:
            self.top_covers[self.count] = cover.content
            self.top_ids[self.count] = movie_id.upper()

        self.count += 1

    def show_top10(self):
        x_offset = 20
        y_offset = 100
        background_width = 1660
        background_height = 3300
        background = Image.new('RGBA', (background_width, background_height), (41, 41, 41, 255))
        assets_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'assets')
        font = ImageFont.truetype(os.path.join(assets_path, 'SmileySans-Oblique.ttf'), 50)

        title_type = ['new_release', 'recent_update', 'today', 'weekly', 'monthly']
        title = Image.open(os.path.join(assets_path, f'{title_type[self.sort_by.value]}.png'))
        title_width = int(title.size[0] / title.size[1] * y_offset)
        title = title.resize((title_width, y_offset))

        for i in range(0, 10):
            img = Image.open(io.BytesIO(self.top_covers[i]))
            img = img.resize((800, 540), Image.LANCZOS)
            x = i % 2 * 800 + x_offset * (i % 2 + 1)
            y = i // 2 * 540 + y_offset * (i // 2 + 1)
            background.paste(img, (x, y))
            draw_id = ImageDraw.Draw(background)
            draw_id.text((x + 10, y + 550), self.top_ids[i], font=font, fill=(255, 153, 0))
            if i == 0:
                background.paste(title, ((background_width - title_width) // 2, 0), title)

        background.save(f'{self.movie_type.name}-{self.sort_by.name}-Top10.png')

    def download(self):
        self.save_path = os.path.join(self.base_save_path, self.sort_by.name)

        if not os.path.exists(self.save_path):
            os.mkdir(self.save_path)

        for page in range(1, 2):
            videos = self.get_all_videos(page)

            for video in videos:
                movie_title = video.text.split('\n')[1]
                movie_id = video.get('alt')
                self.download_cover(movie_title, movie_id)

        self.show_top10()



import os
import m3u8
import ffmpeg
import requests.exceptions
import multiprocessing as mp
from natsort import natsorted
from missav_toolbox.type_enum import MovieType, SortBy
from missav_toolbox.base_downloader import BaseDownloader


class VideoDownloader(BaseDownloader):
    def __init__(self, save_path='videos', movie_type=MovieType.professional, sort_by=SortBy.ReleaseDate, keywords=None):
        super().__init__(save_path, movie_type, sort_by, keywords)

        self.video_id = None

    def download_segment(self, base_link, suffix, segment):
        video_link = base_link + '/' + suffix.split('/')[0] + '/' + segment
        try:
            video = self.scraper.get(video_link, proxies=self.proxy, headers=self.headers)
            video.raise_for_status()
        except requests.exceptions.HTTPError:
            raise requests.HTTPError('Can\'t fetch the content')

        segment = segment.replace('jpeg', 'ts')
        segment_save_path = os.path.join(self.save_path, segment)

        if os.path.exists(segment_save_path):
            return

        print(f'Downloading {segment}')
        with open(segment_save_path, 'wb') as f:
            f.write(video.content)

    def download_video(self, link):
        page_parser = self.get_page_parser(link)
        js = page_parser.find_all('script', type='text/javascript')

        index_begin = js[2].text.find('m3u8')
        index_offset = 112
        uuid = js[2].text[index_begin : index_begin+index_offset].split('|')
        base_link = 'https://surrit.com/' + '-'.join(uuid[i] for i in range(5, 0, -1))

        try:
            m3u8_playlist_content = self.scraper.get(base_link + '/playlist.m3u8', proxies=self.proxy, headers=self.headers)
            m3u8_playlist_content.raise_for_status()
        except requests.exceptions.HTTPError:
            raise requests.HTTPError('Can\'t fetch the content')

        m3u8_playlist = m3u8.loads(m3u8_playlist_content.text)
        suffix = m3u8_playlist.playlists[-1].uri

        try:
            m3u8_video_content = self.scraper.get(base_link + '/' + suffix, proxies=self.proxy, headers=self.headers)
            m3u8_video_content.raise_for_status()
        except requests.exceptions.HTTPError:
            raise requests.HTTPError('Can\'t fetch the content')

        m3u8_video = m3u8.loads(m3u8_video_content.text)
        segments = m3u8_video.segments.uri

        pool = mp.Pool(8)
        for segment in segments:
            pool.apply_async(self.download_segment, args=(base_link, suffix, segment))
        pool.close()
        pool.join()

    def merge_video(self):
        video_output_path = os.path.join(self.save_path, f'{self.video_id}.mp4')
        if os.path.exists(video_output_path):
            return

        video_files = os.listdir(self.save_path)
        video_files = natsorted(video_files)
        video_list_path = os.path.join(self.save_path, 'file_list.txt')

        with open(os.path.join(self.save_path, 'file_list.txt'), 'w') as f:
            for video_file in video_files:
                if video_file == 'file_list.txt':
                    continue
                f.write(f'file {video_file}\n')

        ffmpeg.input(video_list_path, format='concat', safe=0).output(video_output_path, c='copy').run()

    def delete_cache(self):
        caches = os.listdir(self.save_path)
        for cache in caches:
            if not cache.endswith('.mp4'):
                os.remove(os.path.join(self.save_path, cache))

    def download(self):
        video = self.get_all_videos()[0]
        self.video_id = video.get('alt')
        self.save_path = os.path.join(self.base_save_path, self.video_id)
        if not os.path.exists(self.save_path):
            os.mkdir(self.save_path)

        link = video['href']
        self.download_video(link)
        self.merge_video()
        self.delete_cache()



import os
import requests
import cloudscraper
from bs4 import BeautifulSoup
from missav_toolbox.type_enum import MovieType, SortBy


class BaseDownloader:
    def __init__(self, save_path : str, movie_type=MovieType.professional, sort_by=SortBy.ReleaseDate,
                 keywords=None):
        if not os.path.exists(save_path):
            os.makedirs(save_path)

        self.base_save_path = save_path
        self.save_path = None

        if os.environ.get('http_proxy') is not None and os.environ.get('https_proxy') is not None:
            self.proxy = {'http': os.environ.get('http_proxy'), 'https': os.environ.get('https_proxy')}
        else:
            self.proxy = None

        self.movie_type = movie_type
        self.sort_by = sort_by

        self.keywords = self.parse_keywords(keywords)

        self.prefix = 'https://missav.com/dm16/cn/'
        self.types = ['release', 'fc2', 'search/']
        self.suffix = '&page='
        self.all_sort_by = ['?sort=released_at', '?sort=published_at', '?sort=today_views', '?sort=weekly_views',
                            '?sort=monthly_views']

        self.link = f'{self.prefix}{self.types[self.movie_type.value]}{self.keywords}{self.all_sort_by[self.sort_by.value]}'

        self.scraper = cloudscraper.create_scraper(delay=5, browser={'browser': 'chrome',
                                                                     'platform': 'windows',
                                                                     'mobile': False})

        self.headers = {
            "Content-Type": "application/json",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
            'headerser': 'https://google.com'}

    def get_page_parser(self, link, page=None):
        try:
            if page is not None:
                link += f'{self.suffix}{page}'
            content = self.scraper.get(link, proxies=self.proxy, headers=self.headers)
            content.raise_for_status()
        except requests.exceptions.HTTPError:
            raise requests.HTTPError('Can\'t fetch the content')

        return BeautifulSoup(content.text, 'html.parser')

    def get_num_pages(self):
        try:
            num_pages = int(self.get_page_parser(self.link, 1).find_all('a', class_='relative')[-2].contents[0].split('\n')[1])
        except IndexError:
            num_pages = 1

        return num_pages

    def get_all_videos(self, page=1):
        return self.get_page_parser(self.link, page).find_all('a', class_='text-secondary group-hover:text-primary')

    def parse_keywords(self, keywords):
        if keywords is None:
            if self.movie_type == MovieType.search:
                raise ValueError('Keywords should be provided for search type')
            return ''
        elif isinstance(keywords, str):
            self.movie_type = MovieType.search
            return keywords
        elif isinstance(keywords, list):
            self.movie_type = MovieType.search
            return '+'.join(keywords)
        else:
            raise ValueError('Keywords should be a string or a list of strings')

    def download(self):
        pass




from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='missav_toolbox',
    version='1.0.0',
    url='https://github.com/1bananachicken/MissAV-Toolbox',
    packages=find_packages(),
    install_requires=[
        'requests',
        'cloudscraper',
        'Pillow',
        'bs4',
        'm3u8',
        'natsort',
        'ffmpeg-python',
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    author='bananachicken',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
)



from enum import Enum


class MovieType(Enum):
    professional = 0
    fc2 = 1
    search = 2


class SortBy(Enum):
    ReleaseDate = 0
    RecentUpdate = 1
    TodayViews = 2
    WeeklyViews = 3
    MonthlyViews = 4
